(function ($) {
    /*****************************************************************
     * System
     * Constructor
     *****************************************************************/
    function Bookshelf(place, params) {
        params = $.extend(
            true,
            {},
            {
                config: {
                    general:{
                        lang: "en-US",
                        bookshelfversion: 2015.0227,
                        uilang:"fi"
                    },
                    dialogHeight: function() {return Math.floor(window.outerHeight*0.8)},
                    dialogWidth:function() {return Math.max(600, Math.floor(window.outerWidth*0.8))}
                },
                neededPlugins:[
                    "reader"
                ],
                CSSrules: [
                    {"element":" .bookselection",
                    "rules":"display: flex;margin-top: 1em;  flex-wrap: wrap;min-width:150px;"},
                    {"element":"",
                    "rules":"min-width:300px;"},
                    {"element":".removeActive .contentselection .contentSelector:before",
                    "rules":"content:\"X\";width:15px;height:15px;border:1px solid black;border-radius:10px;padding:5px;background-color:red;color:white;font-weight:bold;position: absolute;margin-left: 4.5em;"},
                    {"element":" .contentSelector:hover",
                    "rules":"box-shadow:5px 5px 5px #999,-5px -5px 5px #999;"},
                    {"element":" .contentSelector",
                    "rules":"cursor:pointer;background-color:white; border:1px solid gray;padding:0.2em;border-radius:5px;box-shadow:3px 3px 3px #999;min-height:100px;min-width:100px;display:inline-block;margin-left:1em; "},
                    {"element":" .contentSelector .bookname",
                    "rules":"overflow: hidden; min-width: 90px; min-height: 90px; text-overflow: ellipsis; display: inline-block;"}
                ],
                applicationType: "bookshelf",
                applicationcontentpath: "book",
                contexttype: "book",
                translations: {
                    "en":{
                        "Bookshelf:book":"Book",
                        "Bookshelf:addbook":"Add readerbook",
                        "Bookshelf:bookshelf":"Bookshelf",
                        "Bookshelf:serverimport":"Load book from server",
                        "Bookshelf:close": "Close",
                        "Bookshelf:available_online": "<span class=\"fakelink\" data-fakeurl=\"onlinestudio\">available online until {{{expires}}}</span>"
                    },
                    "sv":{
                        "Bookshelf:book": "Bok",
                        "Bookshelf:bookshelf": "Bokhylla",
                        "Bookshelf:serverimport": "Ladda ner en bok från servern",
                        "Bookshelf:addbook": "Lägg till en bok",
                        "Bookshelf:close": "Stäng",
                        "Bookshelf:available_online": "<span class=\"fakelink\" data-fakeurl=\"onlinestudio\">available online until {{{expires}}}</span>"
                    },
                    "fi":{
                        "Bookshelf:book":"Kirja",
                        "Bookshelf:addbook":"Lisää lukukirja",
                        "Bookshelf:bookshelf":"Kirjahylly",
                        "Bookshelf:serverimport":"Lataa kirja palvelimelta",
                        "Bookshelf:close": "Sulje",
                        "Bookshelf:available_online": "<span class=\"fakelink\" data-fakeurl=\"onlinestudio\">saatavilla verkossa {{{expires}}} asti</span>"
                    }
                }
            },
            params
        );
        
        application.call(this,place,params);
    }
    Bookshelf.prototype =Object.create(application.prototype);
    Bookshelf.prototype.constructor = Bookshelf;
    Bookshelf.prototype.parentClass = application.prototype;
    Bookshelf.prototype.eventHandlerInitParent = Bookshelf.prototype.eventHandlerInit;
    Bookshelf.prototype.mainParent = Bookshelf.prototype.main;
    /******
     * Event handlers
     ******/
    Bookshelf.prototype.eventHandlerInit = function() {
        this.eventHandlerInitParent();
        var bookshelfSelf = this;
        this.place.on('openBook.bookshelf', function(e,data) {
            //console.log(JSON.stringify(jQuery(this)[0]));
        });
        this.place.on('pluginsAvailable.bookshelf', function(e,data) {
            if(data.pluginsAvailable){
                bookshelfSelf.place.trigger('getAvailableContent.system', {path: "book", event: "availableBooks.bookshelf"});
            }else{
                bookshelfSelf.getMissingPlugin(data.missing || bookshelfSelf.neededPlugins);
            };
        });
        // Select book from bookshelf and open it.
        this.place.off('click', '.bookSelector').on('click', '.bookSelector' ,function(e, data) {
            var bookid = $(this).attr('contextid');
            if (bookshelfSelf.place.hasClass('removeActive')) {
                if (jQuery(this).hasClass('contextOpen')) return true;
                if (confirm(bookshelfSelf.config.localizer.localize('system:remove_element_confirm'))) {
                    bookshelfSelf.place.trigger("systemrequest.system", {
                        type: "removeContent",
                        removeFile : {
                            content: "book",
                            contextid: bookid,//.replace(/\.json^/,''),
                            contenttype: "book",
                            contexttype: "book"
                        },
                        callbackEvent: "refreshview.bookshelf",
                        callbacktype: "jQuery"
                    });
                };
            } else {
                bookshelfSelf.openBook(bookid)
            };
        });
        this.place.off('availableBooks.bookshelf').on('availableBooks.bookshelf', function(e,data) {
            if(!data.contentList || data.contentList.length===0){
                bookshelfSelf.books = [];
                bookshelfSelf.place.append("<span>No books available</span>");
            }else{
                bookshelfSelf.books = data.contentList;
                bookshelfSelf.place.trigger('getAvailableViews.system', {event: "availableViews.bookshelf"});
            }
        });
        this.place.off('availableViews.bookshelf').on('availableViews.bookshelf', function(e,data) {
            bookshelfSelf.place.find('.contentselection').remove();
            bookshelfSelf.place.find('.appControlarea .controlpanel').empty();
            var appendPlace = bookshelfSelf.place.append("<div class=\"bookselection contentselection\"></div>").find(".bookselection");
            for(var bookFile in bookshelfSelf.books){
                bookshelfSelf.addcontentSelector(bookshelfSelf.books[bookFile],appendPlace);
            }
            var sorted = appendPlace.find('.contentSelector').sort(
                function (a, b) {
                      var contentA = jQuery(a).attr('title');
                      var contentB = jQuery(b).attr('title');
                      return (contentA < contentB) ? -1 : 1;
                   }
            );
            appendPlace.empty().append(sorted);
            bookshelfSelf.place.trigger('initialized.bookshelf');
            bookshelfSelf.place.trigger(
                'notify.system',
                {msg:"bookshelfDone",type:"notification"}
                
            );
        });
        this.place.off('refreshview').on('refreshview', function() {
            bookshelfSelf.place.trigger('refreshview.bookshelf');
        });
        this.place.off('refreshview.bookshelf').on('refreshview.bookshelf', function() {
            bookshelfSelf.place.empty();
            bookshelfSelf.place.off();
            bookshelfSelf.place.bookshelf(bookshelfSelf.initParams);
        });
        this.place.off('click.addbook').on('click.addbook', '.addbook,.contentSelector.newcontentSelector', function(event) {
            event.stopPropagation();
            bookshelfSelf.place.trigger('newContentinfo.system', {
                contenttype: 'book',
                contexttype: 'book',
                includelocal: true,
                filetype: 'json',
                event: 'addbookinit.ready',
                includeserverdata : true,
                servercontenttype: 'book',
                callbacktype: 'jQuery'
            });
        });
        
        this.place.off('addbookinit.ready').on('addbookinit.ready', function(e,e_data) {
            if(typeof(e_data.detail) !=="undefined"){
                e_data = e_data.detail;
            }
            if (!e_data.success && (e_data.errortype == 12 || e_data.errortype == 13) ){
                bookshelfSelf.place.trigger(e_data.localizableerror+'_error');
                return false;
            };
            if (!e_data.serverdata) {
                bookshelfSelf.openSelectDialog(e_data);
            } else {
                bookshelfSelf.setDataSelectDialog(e_data);
            };
        });
        this.place.off('importContent.bookshelf').on('importContent.bookshelf', function(e, e_data) {
            bookshelfSelf.place.trigger('importContent.system', {
                application: 'bookshelf',
                type: 'book',
                updatetype: 'book',
                event: 'contentCreated.bookshelf',
                content: e_data.contentinfos
            });
        });
        this.place.off('clearDialog.bookshelf').on('clearDialog.bookshelf', function(e,e_data) {
            jQuery.modal.close();
            bookshelfSelf.place.trigger('refreshview.bookshelf');
        });
        this.place.off('contentCreated.bookshelf').on('contentCreated.bookshelf', function(e,e_data) {
            jQuery.modal.close();
            bookshelfSelf.place.trigger('refreshview.bookshelf');
        });
        this.place.off('refreshview.bookshelf').on('refreshview.bookshelf', function() {
            bookshelfSelf.place.empty();
            bookshelfSelf.place.off();
            bookshelfSelf.place.bookshelf(bookshelfSelf.initParams);
        });
    };
    
    /**
     * Open the book
     */
    Bookshelf.prototype.openBook = function(bookid) {
        var uilang = this.config.general.uilang;
        var plugindata = {
            parameters: {
                bookid: bookid,
                contenttype: 'book',
                contentid: bookid,
                contexttype: 'book',
                contextid: bookid
            },
            plugins: [
                {
                    pluginName: 'reader',
                    params: {
                        systemRequests: [
                            {
                                type: 'getContentAsObject',
                                getContentAsObject: {
                                    category: 'content',
                                    type: 'book',
                                    context: {
                                        contextid: bookid,
                                        contexttype: 'book'
                                    },
                                    category: 'content',
                                    type: 'book',
                                    filename: bookid
                                },
                                callbackEvent: 'bookdataReady',
                                relayParams: {
                                    bookid: bookid,
                                    uilang: uilang,
                                    contentId: bookid
                                }
                            }
                        ]
                    }
                }
            ],
            windowAddons: {}
            //, menu: [{
            //    label: 'File',
            //    submenu: [
            //        {label: 'print', click: 'printPage'}
            //    ]
            //}]
        };
        this.place.trigger('startPlugin.system', plugindata);
    };
    
    /**
     * Open book selector dialog
     */
    Bookshelf.prototype.openSelectDialog = function(options) {
        this.dialog = new BookSelectDialog(this.place, options, this.config);
        this.dialog.show();
    };
    
    /**
     * Fill the data on select dialog
     * @param {Object} options   The data for select dialog
     */
    Bookshelf.prototype.setDataSelectDialog = function(options) {
        this.dialog.setOptions(options);
        this.dialog.init();
        this.dialog.updateContainer();
    };

    /******
     * Book selector
     ******/
    Bookshelf.prototype.addcontentSelector = function(fileRef, appPlace) {
        if (fileRef.error) return false;
        var uilang = this.config.general.uilang;
        var bookname = this.sanitize(fileRef.titles && fileRef.titles[uilang]? fileRef.titles[uilang] :
            (fileRef.titles["en"] ? fileRef.titles['en'] :
                (Object.keys(fileRef.titles) ? fileRef.titles[Object.keys(fileRef.titles)[0]]:
                    "Content name missing!")
            ));
        var selector = `
            <span class="contentSelector bookSelector" contexttype="${fileRef.contexttype}" contextid="${fileRef.id}" fileRef="${fileRef.filename}" bookid="${fileRef.id}" title="${bookname}">
                <span class="contentname bookname">${bookname}</span>
            </span>`;
        if (typeof(appPlace) !=="undefined") {
            appPlace.append(selector);
        } else {
            this.place.append(selector);  
        };
    };
    
    /**
     * Get the guided tour for courses.
     * @returns {Object} The guided tour
     */
    Bookshelf.prototype.getGuidedTour = function() {
        var tour = {
            id: 'bookshelftour',
            autostart: true,
            items: [
                {
                    id: 'bookshelf_select',
                    selector: '#applicationControl > .bookshelf .contentSelector.newcontentSelector, #applicationControl > .bookshelf .applicationButton.addbook',
                    side: 'up',
                    position: 'start',
                    title: {
                        en: 'Select books',
                        sv: 'Välj böcker',
                        fi: 'Valitse kirja'
                    },
                    message: {
                        en: 'Books are added on bookshelf by clicking this "plus"-button.',
                        sv: 'Du lägger till böcker till bokhyllan genom att klicka på "plus"-knappen.',
                        fi: 'Kirjoja lisätään kirjahyllyyn "plus"-nappia painamalla.'
                    },
                    enter: [
                        {
                            selector: '#applicationControl > .bookshelf',
                            event: 'scrollintoview'
                        },
                        {
                            selector: '#applicationControl > .bookshelf .contentSelector.newcontentSelector',
                            event: 'scrollintoview'
                        }
                    ],
                    leave: []
                },
                {
                    id: 'bookshelf_booklist',
                    selector: '#simplemodal-data .servernotebooks .serverdatalist',
                    side: 'up',
                    position: 'center',
                    title: {
                        en: 'List of available books',
                        sv: 'Förteckning över tillgängliga böcker',
                        fi: 'Saatavilla olevien kirjojen luettelo'
                    },
                    message: {
                        en: 'This list shows all books that are available for you. Books that you have bought and books that are free. Select the book you want and click OK. Downloading the book might take some time depending on the size of the book.',
                        sv: 'Den här förteckningen visar alla böcker som du har tillgång till. Böcker som du har köpt samt böcker som är tillgängliga gratis. Välj den bok du bill ha klicka på OK. Att ladda ner boken kan ta en liten tid, beroende på hur omfattande boken är.',
                        fi: 'Tämä luettelo näyttää kaikki saatavillasi olevat kirjat. Kirjat, jotka olet ostanut tai jotka ovat ilmaisia. Valitse haluamasi kirja ja klikkaa OK. Kirjan lataaminen saattaa kestää kirjan koosta riippuen jonkin aikaa.'
                    },
                    enter: [
                        {
                            selector: '#applicationControl > .bookshelf .contentSelector.newcontentSelector, #applicationControl > .bookshelf .applicationButton.addbook',
                            event: 'click'
                        }
                    ],
                    leave: [
                        {
                            selector: '.modal-close.simplemodal-close',
                            event: 'click'
                        }
                    ]
                },
                {
                    id: 'bookshelf_open',
                    selector: '#applicationControl > .bookshelf .contentSelector',
                    side: 'up',
                    position: 'start',
                    title: {
                        en: 'Opening a book',
                        sv: 'Öppna en bok',
                        fi: 'Kirjan avaaminen'
                    },
                    message: {
                        en: 'Downloaded books are listed in bookshelf as book icons. Book is opened by clicking the icon.',
                        sv: 'Böcker som har laddats ner visas i bokhyllan som bokikoner. Du öppnar boken genom att klicka på ikonen.',
                        fi: 'Ladatut kirjat ovat lueteltuina kirjahyllyssä kirjakuvakkeina. Kirja avataan klikkaamalla sen kuvaketta.'
                    },
                    enter: [
                        {
                            selector: '#applicationControl > .bookshelf',
                            event: 'scrollintoview'
                        },
                        {
                            selector: '#applicationControl > .bookshelf .contentSelector',
                            event: 'scrollintoview'
                        }
                    ],
                    leave: [
                    ]
                }
            ]
        };
        return tour;
    };
    
    /******
     * Bookshelf main
     ******/
    Bookshelf.prototype.main = function() {
        this.place.trigger('addguidedtour', this.getGuidedTour());
        this.mainParent();
        this.place.find('.controlpanel').append('<span class="applicationButton add-button addbook defaultaddbutton"><span class="buttonIcon"></span><span class="buttonText">'+this.config.localizer.localize('Bookshelf:addbook')+'</span></span>');
    }
    
    { /** jQuery Plugin interface    **/
        var methods = {
            'init' : function(params) {
                return this.each( function() {
                    var bookshelf = new Bookshelf($(this), params);
                    $(this).data('bookshelf', bookshelf);
                });
            },
            'close' : function() {
                return this.each( function() {
                    return $(this).data('bookshelf').close();
                });
            },
            'get' : function(params) {
                return $(this).data('bookshelf')[params];
            },
            'set' : function(params) {
                return $(this).data('bookshelf')[params[0]] = params[1];
            }
        }
        
        $.fn.bookshelf = function( method ) {
            if ( methods[method] ) {
                return methods[method].apply( this, Array.prototype.slice.call( arguments, 1 ));
            } else if ( typeof method === 'object' || ! method ) {
                return methods.init.apply( this, arguments );
            } else {
                $.error( 'Method ' +  method + ' does not exist.' );
                return false;
            }
        }
    }
    
    
    /**
     * Class for book selecting dialog
     * @class BookSelectDialog
     * @constructor
     * @param {jQuery} place    The place for this dialog
     * @param {Object} options  All needed options
     * @param {Object} config   The configs for the dialog
     */
    var BookSelectDialog = function(place, options, config) {
        this.place = place;
        this.config = config;
        this.setOptions(options);
        this.addStyles();
        this.init();
        return this;
    };
    
    /**
     * Set options
     * @param {Object} options   Options for select dialog
     */
    BookSelectDialog.prototype.setOptions = function(options) {
        this.options = $.extend(true, {}, this.defaults, this.options, options);
    };
    
    /**
     * Update container
     */
    BookSelectDialog.prototype.updateContainer = function() {
        this.container.find('#simplemodal-data').html(this.getHtml());
    };
    
    /**
     * Do some initing for the dialog object
     */
    BookSelectDialog.prototype.init = function() {
        var uilang = this.config.general.uilang;
        var localdata = this.options.localdata;
    };
    
    /**
     * Show the book selection dialog
     */
    BookSelectDialog.prototype.show = function() {
        var seldialog = this;
        var localizer = this.config.localizer;
        if (typeof(jQuery.modal) === 'function') {
            this.dialog = jQuery.modal(this.getHtml(), {
                closeHTML: `<a href='#' title='${localizer.localize('Bookshelf:close')}' class='modal-close'>${application.prototype.icons.close}</a>`,
                close: true,
                minHeight: this.config.dialogHeight(),
                maxHeight : this.config.dialogHeight(),
                minWidth: this.config.dialogWidth(),
                autoResize :true,
                onShow: function(dialog){},
                overlayId: undefined,
                containerId: 'bookadd',
                escClose: false
            });
            this.container = this.dialog.d.container;
        };
        this.initHandlers();
        this.updateFiltering()
    };
    
    /**
     * Init the event handlers.
     */
    BookSelectDialog.prototype.initHandlers = function() {
        var selectdialog = this;
        var localizer = this.config.localizer;
        var dialog = this.dialog;
        var container = dialog.d.container;
        container.off('change', 'div.servernotebooks input[type="radio"]').on('change', 'div.servernotebooks input[type="radio"]', function(event) {
            container.trigger('disable_choice', {elementclass: 'addnotebook'}).trigger('enable_ok');
        }).off('enable_ok').on('enable_ok', function(event) {
            event.stopPropagation();
            selectdialog.enableOK();
        }).off('disable_ok').on('disable_ok', function(event) {
            event.stopPropagation();
            selectdialog.disableOK();
        }).off('enable_choice').on('enable_choice', function(event) {
            event.stopPropagation();
            selectdialog.disableOK();
        }).off('disable_choice').on('disable_choice', function(event, data) {
            event.stopPropagation();
            container.find('div.' + data.elementclass).removeClass('elementdisable').find('input').prop('disabled', true);
        }).off('click', '.ok-button').on('click', '.ok-button', function(event) {
            event.stopPropagation();
            if (!$(this).is('.buttondisabled')) {
                selectdialog.place.off('contentCreated.bookshelf').on('contentCreated.bookshelf', function(e, data) {
                    if (typeof(data.detail) !== 'undefined') {
                        data = data.detail;
                    };
                    if (data.success) {
                        selectdialog.place.trigger('clearDialog.bookshelf');
                    } else {
                        content.trigger('registrationerror', {msg: 'registration error'});
                    };
                    selectdialog.place.off('contentCreated.bookshelf');
                });
                let servercontentinfos = [];
                let imports = container.find('div.servernotebooks input[type="radio"]:checked').closest('li.oneserverbook');
                for (let i = 0, len = imports.length; i < len; i++) {
                    let bookindex = imports.eq(i).attr('data-booklistindex');
                    let serverData = selectdialog.options.serverdata && selectdialog.options.serverdata[bookindex] || {};
                    if (typeof(serverData.contextkey) === 'undefined') {
                        serverData.contextkey = 'join_' + serverData.contextid;
                    };
                    servercontentinfos.push(serverData);
                };
                selectdialog.place.trigger('importContent.bookshelf', {contentinfos: servercontentinfos});
                container.append(`<div style="position:absolute;top:0;bottom:0;right:0;left:0;padding:100px;background-color:rgba(255,255,255,0.5);z-index:10;text-align: center;">
                        <div>Loading...</div>
                        <div>${application.prototype.icons.waitdots}</div>
                    </div>
                `);
            };
        });
    };
    
    /**
     * Enable ok-button
     */
    BookSelectDialog.prototype.enableOK = function() {
        this.dialog.d.container.find('.ok-button').removeClass('buttondisabled');
    };
    
    /**
     * Disable ok-button
     */
    BookSelectDialog.prototype.disableOK = function() {
        this.dialog.d.container.find('.ok-button').addClass('buttondisabled');
    };
    
    /**
     * Update filtering
     */
    BookSelectDialog.prototype.updateFiltering = function() {
        
    }
    
    /**
     * Get the html-code for the dialog.
     */
    BookSelectDialog.prototype.getHtml = function() {
        var options = this.options;
        var localizer = this.config.localizer;
        var dialogHTML = `
            <h1>${localizer.localize('Bookshelf:addbook')}</h1>
        `;
        if (options.serverdata && typeof(options.serverdata.length) === 'number') {
            dialogHTML += `
            <div class="servernotebooks">
                <h4>${localizer.localize('Bookshelf:serverimport')}</h4>
                <ul class="serverdatalist">
                ${this.getServerDataList()}
                </ul>
            </div>
            <div class="buttons dialogButtons">
                <span class="applicationButton ok-button buttondisabled">OK</span></div>
            </div>
            `
        } else {
            dialogHTML += `
            <div class="servernotebooks">
                <h4>${localizer.localize('Bookshelf:serverimport')}</h4>
                <ul class="serverdatalist">
                    <li class="oneserverbook" style="text-align: center;">${application.prototype.icons.waitdots}</li>
                </ul>
            </div>
            `;
        };
        return dialogHTML;
    };
    
    /**
     * Get the list of server datas
     */
    BookSelectDialog.prototype.getServerDataList = function() {
        var bookshelf = this;
        var html = [], titles;
        var uilang = this.config.general.uilang;
        var localizer = this.config.localizer;
        var serverdata = this.options.serverdata || [];
        var cdata;
        serverdata.sort(function(a, b) {
            var titlesA, titlesB, titleA = '', titleB = '';
            try {
                titlesA = JSON.parse(a.data).titles;
                titlesB = JSON.parse(b.data).titles;
                titleA = titlesA[uilang] || titlesA['common'];
                titleB = titlesB[uilang] || titlesB['common'];
            } catch (err) {
                console.log(err);
                return -1;
            }
            return (titleA < titleB ? -1 : ((titleB < titleA) ? 1 : 0));
        });
        for (let i = 0, len = serverdata.length; i < len; i++) {
            cdata = JSON.parse(serverdata[i].data);
            titles = cdata.titles || {fi: 'otsikko', en: 'title', sv: 'rubrik'};
            let isLimited = !!(serverdata[i].expires);
            let disabled = (isLimited ? 'disabled="disabled"' : '');
            let expiresStr = '???';
            try {
                let expires = serverdata[i].expires;
                if (typeof(expires) === 'string') {
                    let expd = new Date(expires);
                    if (expires.length > 10) {
                        expiresStr = expd.toLocaleString(uilang);
                    } else {
                        expiresStr = expd.toLocaleDateString(uilang);
                    };
                };
            } catch (err) {
                console.log(err);
            };
            let inOnline = (isLimited ? '(' + localizer.localize('Bookshelf:available_online').replace('{{{expires}}}', expiresStr) + ')' : '');
            html.push(`
                <li class="oneserverbook" data-booklistindex="${i}">
                    <label>
                        ${sanitize(titles[uilang] || titles['common'])}
                        <input type="radio" name="readerselector" ${disabled}>
                    </label>
                    ${inOnline}
                </li>
            `);
        };
        return html.join('\n');
    };
    
    /**
     * Add styles for book select dialog
     */
    BookSelectDialog.prototype.addStyles = function() {
        if ($('head style#bookaddcss').length === 0) {
            $('head').append(`
                <style id="bookaddcss" type="text/css">
                
                </style>
            `);
        };
    };
    
    /**
     * Defaults for book select dialog
     */
    BookSelectDialog.prototype.defaults = {
        
    }

    /**
     * Use the sanitizer of the Bookshelf-class.
     */
    var sanitize = Bookshelf.prototype.sanitize;
    
})(jQuery)