/******
@name NBTrashbin
@version 0.1
@author Petri Salmela <petri.salmela@abo.fi>
@type plugin
@requires jQuery x.x.x or newer
@class NBTrashbin
@description A class and jQuery-plugin for a trashbin on notebook.

TODO:
*******/

/**
 * Requirements:
 * - jQuery
 */

try {
    typeof(jQuery) !== 'undefined';
} catch (err) {
    throw new Error('Missing dependency in ' + err.fileName + '\n' + err);
}

/**
 * Optional requirements
 * - EbookLocalizer
 */

if (typeof(checkOptionalRequirements) !== 'undefined' && checkOptionalRequirements) {
    try {
        typeof(EbookLocalizer) !== 'undefined';
        typeof(ebooklocalizer) !== 'undefined';
    } catch (err) {
        throw new Error('Missing optional dependency in ' + err.fileName + '\n' + err);
    }
}

;(function(window, $, ebooklocalizer){

    /**
     * Helper functions
     */
    
    /**
     * Escape html for security
     */
    var escapeHTML = function(html) {
        return document.createElement('div')
            .appendChild(document.createTextNode(html))
            .parentNode
            .innerHTML
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;')
    };
   
    /**
     * Course note tool for markings, highlightings and notes on notebooks.
     * @constructor
     * @param {jQuery} place    DOM-place for course videoconf
     * @param {Object} options  data for course videoconf
     */
    var NBTrashbin = function(place, options) {
        options = $.extend(true, {}, this.defaults, options);
        this.place = $(place);
        this.init(options);
        this.initHandlers();
        this.initData();
        this.setAttrs();
        this.setStyles();
        this.place.trigger('addguidedtour', this.getTourguide());
        this.show();
        //this.registerConfForm();
        this.appReady();
    }
    
    NBTrashbin.prototype.init = function(options) {
        this.settings = options.settings;
        this.configs = options.configs;
        this.users = this.settings.users;
        this.onepagemode = this.settings.onepagemode;
    };
    
    NBTrashbin.prototype.initData = function() {
        this.appid = this.settings.appid;
        this.apptype = 'nbtrashbin';
        this.place.attr('data-appname', this.appid);
    };
    
    NBTrashbin.prototype.initHandlers = function() {
        var trashbin = this;
        var uilang = this.settings.uilang;
        this.place.on('closechildrenapp', function(event, data){
            event.stopPropagation();
            trashbin.close();
        }).off('closeapp').on('closeapp', function(event, data){
            event.stopPropagation();
            trashbin.close();
        }).off('click', '.trashbin-button[data-action]').on('click', '.trashbin-button[data-action]', function(event, data){
            event.stopPropagation();
            var action = $(this).attr('data-action');
            trashbin.trashplace.find('.trashbin-binapparea').trigger(action);
        }).off('click', '.trashbin-button-removeall').on('click', '.trashbin-button-removeall', function(event, data){
            event.stopPropagation();
            if (confirm(ebooklocalizer.localize('trashbin:areyousure_removeall', uilang))) {
                trashbin.trashplace.find('.trashbin-binapparea').trigger('removeAllTrash');
            };
        }).off('click', '.trashbin-button-guide').on('click', '.trashbin-button-guide', function(event, data){
            event.stopPropagation();
            trashbin.place.trigger('activatetourguide', {tour: 'trashbin', item: 0});
        }).off('change', '.trashbin-categorystatus').on('change', '.trashbin-categorystatus', function(event, data){
            event.stopPropagation();
            var category = $(this).val();
            trashbin.configs.category = category;
            trashbin.refresh();
        }).off('refreshapp').on('refreshapp', function(event, data){
            event.stopPropagation();
            trashbin.refresh();
        }).off('trashbin-pagenumber').on('trashbin-pagenumber', function(event, data){
            event.stopPropagation();
            trashbin.setPagenumber(data);
        }).off('setconfigs').on('setconfigs', function(event, data){
            event.stopPropagation();
            trashbin.setConfigs(data);
        });
    };
    
    NBTrashbin.prototype.setAttrs = function() {
        this.place.addClass('trashbin');
    };
    
    NBTrashbin.prototype.setConfigs = function(data) {
        var trashbin = data.trashbin;
        // TODO
        this.configs = $.extend(this.configs, data);
        this.saveConfigs();
    };
    
    NBTrashbin.prototype.saveConfigs = function(data) {
        // Save config data in parent module
        if (typeof(data) === 'undefined') {
            // Not given data, but save all. (Else save only given data)
            data = {
                type: 'configsave',
                appname: this.appid,
                appid: this.appid,
                apptype: this.apptype,
                configs: JSON.parse(JSON.stringify(this.configs))
            };
        };
        this.place.trigger('saveconfigs', data);
    }
    
    NBTrashbin.prototype.show = function() {
        this.view();
        this.announceTrashbin();
    };
    
    NBTrashbin.prototype.view = function() {
        var uilang = this.settings.uilang;
        var html = this.getTemplate('html');
        this.place.html(html);
        this.drawButtons();
        this.place.find('.trashbin-title').text(ebooklocalizer.localize('trashbin:trashbin', uilang));
        this.trashplace = this.place.find('.trashbin-binarea');
    };
    
    NBTrashbin.prototype.setPagenumber = function(data) {
        this.place.find('.trashbin-pagearea').html(data.pagenum || '');
    }
    
    NBTrashbin.prototype.refresh = function() {
        this.trashplace.find('.trashbin-binapparea').trigger('refresh_trash', {category: this.configs.category});
    };
    
    NBTrashbin.prototype.drawButtons = function() {
        var uilang = this.settings.uilang;
        var buttonset = this.place.find('.trashbin-controlbuttons');
        var buttons = this.controlbuttons;
        var button, buttelem, label;
        for (let i = 0, len = buttons.length; i < len; i++) {
            buttelem = $(this.getTemplate('setbutton'));
            button = buttons[i];
            label = ebooklocalizer.localize(button.label, uilang);
            buttelem.attr('data-action', button.action).attr('data-button-name', button.name).attr('title', label);
            buttelem.html('<span>' + (buttons[i].icon || label) + '</span>');
            buttonset.append(buttelem);
        };
        if (!this.onepagemode) {
            let catbuttonset = this.place.find('.trashbin-categorybuttons');
            let catbuttons = this.categorybuttons;
            let butt, input;
            for (let i = 0, len = catbuttons.length; i < len; i++) {
                buttelem = $(this.getTemplate('catbutton'));
                butt = buttelem.children('.trashbin-button');
                button = catbuttons[i];
                label = ebooklocalizer.localize(button.label, uilang);
                butt.attr('data-action', button.action).attr('data-button-name', button.name).attr('title', label);
                butt.html('<span>' + (button.icon || label) + '</span>');
                input = buttelem.children('input');
                input.attr('value', button.action);
                if (button.checked) {
                    input.attr('checked', 'checked');
                };
                catbuttonset.append(buttelem);
            };
        };
    };
    
    NBTrashbin.prototype.announceTrashbin = function() {
        var binarea = this.place.find('.trashbin-binarea');
        var getTrashbin = function(id, appid) {
            appid = appid || 'noname';
            // TODO
            var appbin = binarea.children('[data-trashbinappid="' + escapeHTML(appid) + '"]');
            if (appbin.length === 0) {
                appbin = $('<div class="trashbin-binapparea" data-trashbinappid="'+escapeHTML(appid)+'"></div>');
                binarea.append(appbin);
            };
            return appbin;
        };
        var clearTrashbin = function(appid) {
            var trashbin = binarea.children('[data-trashbinappid="' + escapeHTML(appid) + '"]');
            trashbin.empty();
        };
        this.place.trigger('trashbinannouncement', [{
            info: {
                appid: this.appid,
                apptype: this.apptype
            },
            data: {
                getTrashbin: getTrashbin,
                clearTrashbin: clearTrashbin
            }
        }]);
    };
    
    NBTrashbin.prototype.registerConfForm = function() {
        var formdata = {apptype: this.apptype, appid: this.appid, confform: this.getConfigForm()};
        this.place.trigger('registerconfigdialog', formdata);
    }
    
    /**
     * Inform outer system that the application is ready
     */
    NBTrashbin.prototype.appReady = function(){
        var uilang = this.settings.uilang;
        this.place.trigger('appready', {apptype: this.apptype, appid: this.appid, title: ebooklocalizer.localize('trashbin:trashbin', uilang)});
    };

    NBTrashbin.prototype.setStyles = function() {
        if ($('head style#nbtrashbinstyles').length === 0) {
            $('head').append('<style type="text/css" id="nbtrashbinstyles">'+this.styles+'</style>')
        }
    };
    
    NBTrashbin.prototype.close = function() {
        this.place.off();
        this.place.trigger('closeappok', {appid: this.appid, apptype: this.apptype});
    };
    
    NBTrashbin.prototype.getConfigForm = function() {
        var uilang = this.settings.uilang;
        var form = {
            name: this.appid,
            title: ebooklocalizer.localize('trashbin:trashbin', uilang),
            icon: this.icons.appicon,
            form: [],
            weight: 8
        };
        
        return form;
    };
    
    NBTrashbin.prototype.defaults = {
        settings: {
            uilang: 'en'
        },
        config: {
            category: 'elements'
        }
    };
    
    NBTrashbin.prototype.styles = [
        '.virumcourse-appcontent.trashbin {background-color: #eee; font-family: helvetica, Arial, sans-serif; padding: 1em; display: flex; flex-direction: column; justify-content: flex-start; height: 96%; box-sizing: border-box;}',
        '.trashbin-header {flex-shrink: 0; flex-grow: 0;}',
        '.trashbin-iconarea {display: inline-block;}',
        '.trashbin-binarea {margin: 0; margin-right: 1.5em; flex-shrink: 1; flex-grow: 1; overflow-y: auto; padding-right: 30px;}',
        '.trashbin-buttonbar {display: flex; flex-direction: row; flex-shrink: 0; flex-grow: 0;}',
        '.trashbin-binapparea {position: relative;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper {background-color: white; padding: 1.6em 0.8em 0.5em; min-height: 3em; margin: 0.8em 10px; box-shadow: 0 0 4px rgba(0,0,0,0.5); transition: box-shadow 0.4s; float: none!important; max-width: none!important; width: auto!important;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper:hover {box-shadow: 2px 2px 30px rgba(0,0,0,0.4);}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper::before {content: attr(data-modifier); display: block; position: absolute; top: 0; left: 0; padding: 0.2em 0.3em; background-color: #eee; border-right: 1px solid #888; border-bottom: 1px solid #888; border-radius: 0 0 0.3em 0;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper::after {content: attr(data-modified); display: block; position: absolute; top: 0; right: 0; padding: 0.2em 0.3em; background-color: #eee; border-left: 1px solid #888; border-bottom: 1px solid #888; border-radius: 0 0 0 0.3em;}',
        '.trashbin-binapparea .trashbin-page-modifier {display: block; position: absolute; top: 0; left: 0; padding: 0.2em 0.3em; background-color: #eee; border-right: 1px solid #888; border-bottom: 1px solid #888; border-radius: 0 0 0.3em 0;}',
        '.trashbin-binapparea .trashbin-page-modified {display: block; position: absolute; top: 0; right: 0; padding: 0.2em 0.3em; background-color: #eee; border-left: 1px solid #888; border-bottom: 1px solid #888; border-radius: 0 0 0 0.3em;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > div {float: none;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-iconbar {width: 30px;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper:hover > .elementset-iconbar {opacity: 1;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-iconbar .elementset-iconbar-icon[data-icon-action="gotolink"] {display: none;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-iconbar .elementset-iconbar-icon[data-icon-action="copyelement"] {display: none;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-iconbar .elementset-iconbar-icon[data-icon-action="moveelement"] {display: block;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-iconbar .elementset-iconbar-icon[data-icon-action="remove_from_trash"] {cursor: pointer;}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-iconbar .elementset-iconbar-icon[data-icon-action="remove_from_trash"]:hover {background-color: rgba(255,255,255,0.4);}',
        '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-iconbar .elementset-iconbar-icon svg {width: 30px; height: 30px;}',
        '.trashbin-binapparea > .ebook-elementset .sdeditorwrapper ul.sdeditor-menubar {display: none;}',
        '.trashbin-binapparea[data-trashbin-category="pages"] {background-color: white; margin: 5px; box-shadow: 0 0 4px rgba(0,0,0,0.5);}',
        '.trashbin-iconbar {z-index: 10; position: absolute; top: 2em; right: -30px; background-color: rgba(230,230,230,0.8); border: 1px solid #aaa; border-radius: 4px;}',
        '.trashbin-iconbar .trashbin-iconbar-icon {padding: 3px; transition: background-color 0.2s;}',
        '.trashbin-iconbar .trashbin-iconbar-icon[data-icon-action="movepage"] {cursor: move;}',
        '.trashbin-iconbar .trashbin-iconbar-icon[data-icon-action="remove_from_trash"] {cursor: pointer;}',
        '.trashbin-iconbar .trashbin-iconbar-icon:hover {bacground-color: rgba(255,255,255,0.5);}',
        '.trashbin-iconbar .trashbin-iconbar-icon svg path {fill: black;}',
        '.elementset-iconbar .elementset-iconbar-icon svg path {fill: black;}',
        '.trashbin-binapparea > .ebook-pageelement .elementset-iconbar .elementset-iconbar-icon[data-icon-action="gotolink"] {display: none;}',
        '.trashbin-header h1 {padding: 1em 0.3em 0; border-bottom: 3px solid black;}'
    ].join('\n');
    
    NBTrashbin.prototype.icons = {
        appicon: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-trashbin"><path style="stroke: none;" d="M6.5 3 h7 v-1 h3 v1 h7 a1.5 1.5 0 0 1 0 3 h-17 a1.5 1.5 0 0 1 0 -3z M5 7 h20 v20 a3 3 0 0 1 -3 3 h-14 a3 3 0 0 1 -3 -3z m3 5 v13 a1.5 1.5 0 0 0 3 0 v-13 a1.5 1.5 0 0 0 -3 0z m5.5 0 v13 a1.5 1.5 0 0 0 3 0 v-13 a1.5 1.5 0 0 0 -3 0z m5.5 0 v13 a1.5 1.5 0 0 0 3 0 v-13 a1.5 1.5 0 0 0 -3 0z "></path></svg>',
        help: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-question mini-icon-help"><circle class="mini-icon-background" stroke="none" fill="none" cx="15" cy="15" r="15"></circle><path class="mini-icon-foreground" style="stroke: none;" d="M8 11 c2 -9 14 -8 14 0 c0 4 -5 6 -5 8 v1 h-4 v-1 c0 -4 5 -5 5 -8 c0 -3 -5 -4 -6 1z m5 11 h4 v4 h-4z"></path></svg>',
        next: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="-3 -3 36 36" class="mini-icon mini-icon-right"><path class="mini-icon-extrapath" style="stroke: none;" opacity="0" d="M15 -2 a17 17 0 0 1 0 34 a17 17 0 0 1 0 -34z m0 2 a 15 15 0 0 0 0 30 a15 15 0 0 0 0 -30z"></path><path style="stroke: none;" d="M10 5 l15 10 l-15 10z"></path></svg>',
        prev: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="-3 -3 36 36" class="mini-icon mini-icon-left"><path class="mini-icon-extrapath" style="stroke: none;" opacity="0" d="M15 -2 a17 17 0 0 1 0 34 a17 17 0 0 1 0 -34z m0 2 a 15 15 0 0 0 0 30 a15 15 0 0 0 0 -30z"></path><path style="stroke: none;" d="M19 25 l-15 -10 l15 -10z"></path></svg>',
        shredder: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-shredder"><path style="stroke: none;" d="M7 10 v-8 h12 l4 4 v4 h-1 v-4 h-3 v-3 h-11 v7z m3 -5 h8 v1 h-8z m0 3 h10 v1 h-10z m-5 3 h20 a2 2 0 0 1 2 2 v2 a2 2 0 0 1 -2 2 h-2 v-2 h-16 v2 h-2 a2 2 0 0 1 -2 -2 v-2 a2 2 0 0 1 2 -2z m3 5 h2 v10 h-2z m3 0 h2 v8 h-2z m3 0 h2 v11 h-2z m3 0 h2 v9 h-2z m3 0 h2 v10 h-2z"></path></svg>'
    }
    
    NBTrashbin.prototype.getTemplate = function(id) {
        var uilang = this.settings.uilang;
        var templates = {
            html: `
                <h1 class="trashbin-header">
                    <div class="trashbin-iconarea"><div class="trashbin-icon">${this.icons.appicon}</div></div>
                    <span class="trashbin-title"></span>
                </h1>
                <div class="trashbin-buttonbar">
                    <div class="trashbin-controlbuttons ffwidget-buttonset ffwidget-horizontal"></div>
                    <div class="trashbin-categorybuttons ffwidget-buttonset ffwidget-horizontal"></div>
                    <div class="trashbin-buttons ffwidget-buttonset ffwidget-horizontal">
                        <button class="trashbin-button-removeall ffwidget-setbutton">${this.icons.shredder}</button><button class="trashbin-button-guide ffwidget-setbutton">${this.icons.help}</button>
                    </div>
                </div>
                <div class="trashbin-pagearea"></div>
                <div class="trashbin-binarea"></div>
            `,
            setbutton: `<button class="trashbin-button ffwidget-setbutton"></button>`,
            catbutton: `<label class="ffwidget-setbutton-wrapper"><input type="radio" name="trashbin-categorystatus" class="trashbin-categorystatus" /><span class="trashbin-button ffwidget-setbutton"></span></label>`
        };
        return templates[id];
    };
    
    /**
     * Control buttons
     */
    NBTrashbin.prototype.controlbuttons = [
        {
            name: 'prevpage',
            label: 'trashbin:prevpage',
            icon: NBTrashbin.prototype.icons.prev,
            action: 'trashbin-prevpage'
            
        },
        {
            name: 'nextpage',
            label: 'trashbin:nextpage',
            icon: NBTrashbin.prototype.icons.next,
            action: 'trashbin-nextpage'
        }
    ];
    
    /**
     * Category buttons
     */
    NBTrashbin.prototype.categorybuttons = [
        {
            name: 'elements',
            label: 'trashbin:elements',
            icon: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-image"><path style="stroke: none;" d="M3 6 h24 v18 h-24z m2 2 v14 h20 v-14z m1 12 l5 -5 l3 3 l6 -6 l4 4 v2 l-4 -4 l-6 6 l-3 -3 l-5 5z"></path></svg>',
            action: 'elements',
            checked: true
            
        },
        {
            name: 'pages',
            label: 'trashbin:pages',
            icon: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-pageelement"><path stroke="none" fill="black" d="M4 2 h22 v20 a8 8 0 0 1 -6 6 h-16 0z m1 1 v24 h15 v-5 h5 v-19z m3 8 h12 v1 h-12z m0 3 h12 v1 h-12z m0 3 h12 v1 h-12z m0 3 h12 v1 h-12z"></path></svg>',
            action: 'pages'
        }
    ];
    
    /**
     * Guided tour for MarginNotes
     */
    NBTrashbin.prototype.getTourguide = function() {
        let tourguide = {
            id: 'trashbin',
            autostart: true,
            items: [
                {
                    id: 'trashbin',
                    selector: '.virumcourse-applistitem[data-apptype="trashbin"]',
                    side: 'left',
                    position: 'start',
                    title: {
                        en: 'Trashbin',
                        fi: 'Roskakori',
                        sv: 'Skräpkorg'
                    },
                    message: {
                        en: 'From trashbin the user can find and recover recently removed content items.',
                        fi: 'Roskakorista käyttäjä voi löytää ja palauttaa poistettua sisältöä.',
                        sv: 'Via skräpkorgen kan användaren återställa tidigare raderade element.'
                    },
                    enter: [
                        {
                            selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                            event: 'click'
                        }
                    ],
                    leave: [
                        {
                            selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                            event: 'click'
                        }
                    ]
                }
            ]
        };
        if (!this.onepagemode) {
            // We have multiple pages!
            tourguide.items.push(
                {
                    id: 'trashbin-category',
                    selector: '.trashbin-categorybuttons',
                    side: 'up',
                    position: 'center',
                    title: {
                        en: 'Trashbin category',
                        fi: 'Roskakorin luokat',
                        sv: 'Soptunna, kategori'
                    },
                    message: {
                        en: 'User can select to view either deleted <strong>elements</strong> or <strong>pages</strong>.',
                        fi: 'Käyttäjä voi valita, näytetäänkö poistetut <strong>elementit</strong> vai poistetut <strong>sivut</strong>.',
                        sv: 'Användaren kan överblicka antingen raderade <strong>element</strong> eller raderade <strong>sidor</strong>.'
                    },
                    enter: [
                        {
                            selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                            event: 'click'
                        }
                    ],
                    leave: [
                        {
                            selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                            event: 'click'
                        }
                    ]
                }
            );
        };
        tourguide.items.push(
            {
                id: 'trashbin-binarea-elements',
                selector: '.trashbin-binapparea > .ebook-elementset, .trashbin-binapparea',
                side: 'left',
                position: 'start',
                title: {
                    en: 'List of removed elements',
                    fi: 'Poistettujen elementtien lista',
                    sv: 'Lista över raderade element'
                },
                message: {
                    en: 'Deleted elements are show as a list. Maximum of 20 elements at a time.',
                    fi: 'Poistetut elementit näytetään listana, enintään 20 kerrallaan.',
                    sv: 'Raderade element visas i en lista. Som mest visas 20 element samtidigt.'
                },
                enter: [
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                        event: 'click'
                    },
                    {
                        selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                        event: 'click'
                    }
                ],
                leave: [
                    {
                        selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                        event: 'click'
                    },
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                        event: 'click'
                    }
                ]
            }
        );
        if (!this.onepagemode) {
            // We have multiple pages!
            tourguide.items.push({
                id: 'trashbin-binarea-pages',
                selector: '.trashbin-binapparea, .trashbin-binapparea',
                side: 'up',
                position: 'start',
                title: {
                    en: 'Removed elements',
                    fi: 'Poistetu sivut',
                    sv: 'Raderade element'
                },
                message: {
                    en: 'Each removed page is shown on it\'s own.',
                    fi: 'Jokainen poistettu sivu näytetään erikseen.',
                    sv: 'Varje raderad sida visas skilt för sig.'
                },
                enter: [
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                        event: 'click'
                    },
                    {
                        selector: '.trashbin-categorybuttons .trashbin-button[data-action="pages"]',
                        event: 'click'
                    }
                ],
                leave: [
                    {
                        selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                        event: 'click'
                    },
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                        event: 'click'
                    }
                ]
            });
        };
        tourguide.items.push(
            {
                id: 'trashbin-navi',
                selector: '.trashbin-controlbuttons',
                side: 'up',
                position: 'start',
                title: {
                    en: 'Browsing of the trashbin',
                    fi: 'Roskakorin selaaminen',
                    sv: 'Genomgång av skräpkorgen'
                },
                message: {
                    en: 'If there are lots of deleted elements or pages, they can be browsed by forward and backward navigation. Below the buttons the total number of pages and the current page are shown.',
                    fi: 'Jos poistettuja elementtejä on paljon, voi niitä selata navigoimalla sivuja eteen ja taakse päin. Nappuloiden alla on näkyvissä sivujen kokonaismäärä ja nykyisen sivun numero.',
                    sv: 'Om det finns många raderade element, kan de gås igenom med framåt- och bakåtknapparna. Under knapparna syns antalet sidor och den nuvarande sidans nummer.'
                },
                enter: [
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                        event: 'click'
                    }
                ],
                leave: [
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                        event: 'click'
                    }
                ]
            },
            {
                id: 'trashbin-move-element',
                selector: '.trashbin-binapparea .elementset-iconbar-icon[data-icon-action="moveelement"]',
                side: 'up',
                position: 'end',
                title: {
                    en: 'Recover element from trash',
                    fi: 'Elementin palauttaminen',
                    sv: 'Återställ element'
                },
                message: {
                    en: 'An element can be recovered from trash by dragging it to the selected page (in edit mode).',
                    fi: 'Elementin voi palauttaa roskakorista raahaamalla sen valitulle muokkaustilassa olevalle sivulle.',
                    sv: 'Ett element kan återställas från skräpkorgen genom att dra det till en vald sida (i editeringsläge).'
                },
                enter: [
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                        event: 'click'
                    },
                    {
                        selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                        event: 'click'
                    },
                    {
                        selector: '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-element',
                        event: 'focus'
                    }
                ],
                leave: [
                    {
                        selector: '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-element',
                        event: 'blur'
                    },
                    {
                        selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                        event: 'click'
                    },
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                        event: 'click'
                    }
                ]
            },
            {
                id: 'trashbin-remove-element',
                selector: '.trashbin-binapparea .elementset-iconbar-icon[data-icon-action="remove_from_trash"]',
                side: 'up',
                position: 'end',
                title: {
                    en: 'Remove element from trash',
                    fi: 'Elementin poistaminen',
                    sv: 'Ta bort element permanent'
                },
                message: {
                    en: 'A single element can be permanently removed from trash by clicking it\'s shredder icon.',
                    fi: 'Elementin voi poistaa roskakorista lopullisesti klikkaamalla sen kohdalla silppurikuvaketta.',
                    sv: 'Ett enskilt element kan tas bort permanent genom att klicka på dokumentförstörar-ikonen.'
                },
                enter: [
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                        event: 'click'
                    },
                    {
                        selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                        event: 'click'
                    },
                    {
                        selector: '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-element',
                        event: 'focus'
                    }
                ],
                leave: [
                    {
                        selector: '.trashbin-binapparea > .ebook-elementset > .ebook-elementset-body > .elementset-elementwrapper > .elementset-element',
                        event: 'blur'
                    },
                    {
                        selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                        event: 'click'
                    },
                    {
                        selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                        event: 'click'
                    }
                ]
            }
        );
        if (!this.onepagemode) {
            // We have multiple pages!
            tourguide.items.push(
                {
                    id: 'trashbin-move-page',
                    selector: '.trashbin-binapparea .trashbin-iconbar-icon[data-icon-action="movepage"]',
                    side: 'up',
                    position: 'end',
                    title: {
                        en: 'Recover page from trash',
                        fi: 'Sivun palauttaminen',
                        sv: 'Återställ sida'
                    },
                    message: {
                        en: 'A page can be recovered by dragging it on the selected page (in edit mode). The page will be added after that page.',
                        fi: 'Sivun voi palauttaa raahaamalla sen valitun muokkaustilassa olevan sivun päälle. Sivu lisätään valittua sivua seuraavaksi sivuksi.',
                        sv: 'En sida kan återställas genom att dra den till en vald sida (i editeringsläge). Sidan läggs till efter den valda sidan.'
                    },
                    enter: [
                        {
                            selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                            event: 'click'
                        },
                        {
                            selector: '.trashbin-categorybuttons .trashbin-button[data-action="pages"]',
                            event: 'click'
                        }
                    ],
                    leave: [
                        {
                            selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                            event: 'click'
                        },
                        {
                            selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                            event: 'click'
                        }
                    ]
                },
                {
                    id: 'trashbin-remove-page',
                    selector: '.trashbin-binapparea .trashbin-iconbar-icon[data-icon-action="remove_from_trash"]',
                    side: 'up',
                    position: 'end',
                    title: {
                        en: 'Remove page from trash',
                        fi: 'Sivun poistaminen',
                        sv: 'Ta bort sida permanent'
                    },
                    message: {
                        en: 'Page can removed from the trash by clicking the shredder icon.',
                        fi: 'Sivun voi poistaa lopullisesti klikkaamalla sen kohdalla silppurikuvaketta.',
                        sv: 'En sida kan tas bort permanent genom att klicka på dokumentförstörar-ikonen.'
                    },
                    enter: [
                        {
                            selector: '.virumcourse-applistitem[data-apptype="trashbin"]:not(.tabactive)',
                            event: 'click'
                        },
                        {
                            selector: '.trashbin-categorybuttons .trashbin-button[data-action="pages"]',
                            event: 'click'
                        }
                    ],
                    leave: [
                        {
                            selector: '.trashbin-categorybuttons .trashbin-button[data-action="elements"]',
                            event: 'click'
                        },
                        {
                            selector: '.virumcourse-applistitem[data-apptype="trashbin"].tabactive',
                            event: 'click'
                        }
                    ]
                }
            );
        };
        return tourguide;
    };
    
    NBTrashbin.localization = {
        "en": {
            "trashbin:trashbin": "Trashbin",
            "trashbin:nextpage": "Next page",
            "trashbin:prevpage": "Previous page",
            "trashbin:elements": "Elements",
            "trashbin:pages": "Pages",
            "trashbin:areyousure_remove": "You are removing this element or page from trashbin.\nAre you sure?",
            "trashbin:areyousure_removeall": "You are removing all elements or pages, that are visible on this view, from trashbin.\nAre you sure?"
        },
        "fi": {
            "trashbin:trashbin": "Roskakori",
            "trashbin:nextpage": "Seuraava sivu",
            "trashbin:prevpage": "Edellinen sivu",
            "trashbin:elements": "Elementit",
            "trashbin:pages": "Sivut",
            "trashbin:areyousure_remove": "Olet poistamassa tämän elementin tai sivun roskakorista.\nOletko varma?",
            "trashbin:areyousure_removeall": "Olet poistamassa kaikki näkyvissä olevat elementit tai sivut roskakorista.\nOletko varma?"
        },
        "sv": {
            "trashbin:trashbin": "Soptunna",
            "trashbin:nextpage": "Nästa sida",
            "trashbin:prevpage": "Föregående sida",
            "trashbin:elements": "Elementer",
            "trashbin:pages": "Sidor",
            "trashbin:areyousure_remove": "Du håller på och tar bort element från skräpkorgen. \nÄr du säker?",
            "trashbin:areyousure_removeall": "Du håller på och tar bort samtliga element på denna sida från skräpkorgen. \nÄr du säker?"
        }
    }

    if (ebooklocalizer) {
        ebooklocalizer.addTranslations(NBTrashbin.localization);
    } else {
        ebooklocalizer = {
            translations: {},
            addTranslations: function(trans){
                this.translations = $.extend(true, this.translations, trans);
            },
            localize: function(key, lang){
                lang = (this.translations[lang] ? lang : 'en');
                return this.translations[lang] && this.translations[lang][key] || key;
            }
        }
        ebooklocalizer.addTranslations(NBTrashbin.localization);
    }
    
    /**** jQuery-plugin *****/
    var methods = {
        'init': function(params){
            return this.each(function(){
                var trash = new NBTrashbin(this, params);
            });
        },
        'getdata': function(){
            var $place = $(this).eq(0);
            $place.trigger('getdata');
            var data = $place.data('[[trashbindata]]');
            return data;
        },
        'geticon': function() {
            return NBTrashbin.prototype.icons.appicon;
        },
        'gettitle': function() {
            return '';
        }
    }
    
    $.fn.nbtrashbin = function(method){
        if (methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof(method) === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error('Method ' + method + ' does not exist in trashbin.');
            return false;
        }
    };

    
})(window, jQuery, window.ebooklocalizer);
