/**
 * QuillElement
 *
 * @author Petri Salmela <petri.salmela@fourferries.fi>
 * Copyright: Four Ferries Oy
 */
/******
@name QuillElement
@version 0.1
@author Petri Salmela <petri.salmela@fourferries.fi>
@type plugin
@requires jQuery x.x.x or newer
@class QuillElement
@description A class and jQuery-plugin for a text element with Quill as editor.

TODO:
*******/

/**
 * Requirements:
 * - jQuery
 * - Quill
 */

try {
    typeof(jQuery) === 'undefined' && jQuery;
    typeof(Quill) === 'undefined' && Quill;
} catch (err) {
    throw new Error('Missing dependency in ' + err.fileName + '\n' + err);
}

//try {
//    typeof(ebooklocalizer) === 'undefined' && ebooklocalizer;
//} catch (err) {
//    var ebooklocalizer;
//};

(function(window, $, ebooklocalizer) {

    /**** jQuery-plugin *****/
    var methods = {
        'init': function(params){
            return this.each(function(){
                let module = new QuillElement(this, params);
            });
        },
        'getdata': function(){
            var $place = $(this).eq(0);
            $place.trigger('getdata');
            var data = $place.data('[[elementdata]]');
            return data;
        }
    }
    
    $.fn.quillelement = function(method){
        if (methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof(method) === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error('Method ' + method + ' does not exist in QuillElement.');
            return false;
        };
    };
    
    /**
     * Quill settings in edit mode
     */
    var QuillConfigsEdit = window.QuillConfigsEdit = {
        theme: 'snow',
        placeholder: 'Write some text',
        //debug: 'info',
        //readOnly: true,
        modules: {
            toolbar: {
                container: [
                    [{'header': [1, 2, false]}],
                    ['bold', 'italic', 'underline', 'strike'],
                    [{'list': 'ordered'}, {'list': 'bullet'}, 'blockquote', 'code-block'],
                    [{'align': []}],
                    ['link', 'mathquill', {'script': 'sub'}, {'script': 'super'}],
                    [{'color': []}, {'background': []}],
                    //['image'],
                    ['clean']
                ],
                handlers: {
                }
            }
            //toolbar: [
            //    [{'header': [1, 2, false]}],
            //    ['bold', 'italic', 'underline'],
            //    [{'list': 'ordered'}, {'list': 'bullet'}],
            //    ['blockquote', 'code-block'],
            //    [{'script': 'sub'}, {'script': 'super'}]
            //    [{'size': ['small', false, 'large', 'huge']}],
            //    [{'color': []}, {'background': []}],
            //    [{'font': []}, 'formula', 'image'],
            //    [{'align': []}],
            //    ['link', 'clean']
            //],
            //keyboard: {
            //    bindings: {
            //        oma: {
            //            key: 'M',
            //            ctrlKey: true,
            //            handler: function(range, context) {
            //                //alert('Matikkaa!');
            //                console.log('context', context);
            //                let delta = quill.insertEmbed(range.index, 'mathquill', {
            //                    latex: ''
            //                }, Quill.sources.USER);
            //                console.log('delta', delta);
            //                quill.setSelection(range.index + 1, Quill.sources.SILENT);
            //            }
            //        }
            //    }
            //}
        }
    };
    
    /**
     * Quill settings in view mode
     */
    var QuillConfigsView = window.QuillConfigsView = {
        theme: 'snow',
        readOnly: true,
        //debug: 'info',
        modules: {
            toolbar: false,
        }
    };
    
    /**
     * If MathQuill is available, use it for formulas.
     * - Init the MathQuill interface
     * - Add MathQuill support into Quill
     */
    try {
        if (typeof(MathQuill) === 'function' && MathQuill){
            
            //if (typeof(window.MQ) === 'undefined') {
                // Init MathQuill interface
                var MQ = MathQuill.getInterface(2);
                var MQConfigs = {
                    spaceBehavesLikeTab: true,
                    autoCommands: 'pi alpha beta gamma delta theta sqrt sum',
                    autoOperatorNames: 'sin cos tan ln log lg lim arcsin arccos arctan',
                    handlers: {
                        moveOutOf: function(direction, mathField) {
                            switch (direction) {
                                case MQ.R:  // Right
                                    $(mathField.el()).trigger('getOutMathquillR');
                                    break;
                                case MQ.L:  // Left
                                    $(mathField.el()).trigger('getOutMathquillL');
                                    break;
                                default:
                                    break;
                            }
                        },
                        edit: function(mathField) {
                            // Update the edited latex to .ql-mathquill -element's attribute.
                            let latex = mathField.latex();
                            let parent = $(mathField.el()).closest('.ql-mathquill');
                            parent.attr('data-value', latex);
                        }
                    }
                };
                MQ.config(MQConfigs);
            //} else {
                //var MQ = window.MQ;
            //};
            
            // Add the handler for Quill toolbar's MathQuill-button
            QuillConfigsEdit.modules.toolbar.handlers.mathquill = function() {
                let quill = this.quill;
                quill.focus();
                let range = quill.getSelection();
                let delta = quill.insertEmbed(range.index, 'mathquill', {
                    latex: '',
                    setFocus: true
                }, Quill.sources.USER);
                quill.setSelection(range.index + 1, Quill.sources.SILENT);
            };
            
            // Add the handler for ctrl/cmd+m for adding MathQuill
            QuillConfigsEdit.modules.keyboard = {
                bindings: {
                    math: {
                        key: 'M',
                        shortKey: true,
                        handler: function() {
                            let quill = this.quill;
                            quill.focus();
                            let range = quill.getSelection();
                            let delta = quill.insertEmbed(range.index, 'mathquill', {
                                latex: '',
                                setFocus: true
                            }, Quill.sources.USER);
                            quill.setSelection(range.index + 1, Quill.sources.SILENT);
                        }
                    }
                }
            };
            
            // Create a new MqBlot for representing MathQuill in Quill.
            let InlineEmbed = Quill.import('blots/embed');
            class MqBlot extends InlineEmbed {
                static create(value) {
                    let node = super.create();
                    let setFocus = !!value.setFocus;
                    let classes = [];
                    if (setFocus) {
                        delete value.setFocus;
                        classes.push('quillelement-mq-setfocus');
                    };
                    node.innerHTML = `<span class="mq-mathfield ${classes.join(' ')}">${value.latex || ''}</span>`;
                    node.setAttribute('data-value', value.latex);
                    setTimeout(function() {$(node).trigger('rendermathquill');}, 0);
                    return node;
                };
                
                static value(node) {
                    let mqnode = $(node).find('.mq-mathfield').get(0);
                    let mqfield = MQ(mqnode);
                    return {
                        latex: mqfield && mqfield.latex()  || node.getAttribute('data-value')
                    }
                };
            };
            MqBlot.blotName = 'mathquill';
            MqBlot.className = 'ql-mathquill';
            MqBlot.tagName = 'SPAN';
            // Use the default 'formula' icon for MathQuill toolbar button.
            Quill.imports['ui/icons'].mathquill = Quill.imports['ui/icons'].formula;
            Quill.register(MqBlot);
        };
    } catch (err) {
        console.log('QuillElement: MathQuill is not available.');
    }

    /**
     * If highlight.js is available, use syntax highlighting.
     */
    try {
        if (typeof(hljs) === 'object' && hljs) {
            //QuillConfigsEdit.modules.syntax = true;
            QuillConfigsView.modules.syntax = true;
        };
    } catch (err) {
        console.log('QuillElement: highlight.js is not available.')
    };

    /**
     * Quill element to show and edit text element.
     * @requires quill.js   Quill library
     * @class QuillElement
     * @constructor
     * @param {jQuery} place   The place for the element
     * @param {Object} options The data for the element
     */
    var QuillElement = function(place, options) {
        options = $.extend(true, {}, this.defaults, options);
        this.type = 'quillelement';
        this.settings = options.settings;
        this.metadata = options.metadata;
        this.metadata.creator = this.metadata.creator || this.settings.username;
        this.metadata.created = this.metadata.created || (new Date()).getTime();
        this.data = options.data;
        this.place = $(place);
        this.place.addClass('quillelement');
        if ($('head style#quillelementstyle').length === 0) {
            $('head').append(this.styles);
        };
        this.init();
        window.quillelement = this;
    };
    
    QuillElement.SAVEDELAY = 2000;
    
    QuillElement.prototype.init = function() {
        this.setMode(this.settings.mode);
        this.show();
    }
    
    /**
     * Set the mode
     * @param {String} mode    The mode of the element ('view'|'edit'|'review'|'author')
     */
    QuillElement.prototype.setMode = function(mode) {
        if (!this.modes[mode]) {
            mode = 'view';
        };
        this.settings.mode = mode;
        var modedata = this.modes[mode];
        this.editable = modedata.editable;
        this.reviewable = modedata.reviewable;
    }
    
    /**
     * Show the element
     */
    QuillElement.prototype.show = function() {
        this.initHandlers();
        if (this.editable) {
            this.place.addClass('editmode').attr('data-elementmode', this.settings.mode);
            this.edit();
        } else {
            this.place.removeClass('editmode').attr('data-elementmode', this.settings.mode);
            this.view();
        };
    };
    
    /**
     * Show the element in view mode
     */
    QuillElement.prototype.view = function() {
        if (this.editorPlace) {
            this.editorPlace.remove();
        };
        this.place.html(`
            <div class="quillelement-editor"></div>
        `);
        this.editorPlace = this.place.children('.quillelement-editor');
        this.quill = new Quill(this.editorPlace.get(0), QuillConfigsView);
        this.addMatchers();
        this.quill.setContents(this.data.delta);
    };
    
    /**
     * Show the element in edit mode
     */
    QuillElement.prototype.edit = function() {
        var self = this;
        if (this.editorPlace) {
            this.editorPlace.remove();
        };
        this.place.html(`
            <div class="quillelement-editor"></div>
        `);
        this.editorPlace = this.place.children('.quillelement-editor');
        if ($('.notebook-viewarea').length > 0) {
            QuillConfigsEdit.scrollingContainer = '.notebook-viewarea';
        };
        this.quill = new Quill(this.editorPlace.get(0), QuillConfigsEdit);
        this.addMatchers();
        this.quill.setContents(this.data.delta);
        this.quill.on('text-change', function(delta, oldDelta, source) {
            if (source === 'user') {
                self.place.trigger('quillchanged');
            };
        });
    };
    
    /**
     * Remove all handlers
     */
    QuillElement.prototype.removeHandlers = function() {
        this.place.off();
    };
    
    /**
     * Init event handlers
     */
    QuillElement.prototype.initHandlers = function() {
        this.removeHandlers();
        this.initHandlersCommon();
        if (this.editable) {
            this.initHandlersEdit();
        } else {
            this.initHandlersView();
        };
    };
    
    /**
     * Init event handlers for all modes
     */
    QuillElement.prototype.initHandlersCommon = function() {
        var self = this;
        this.place.off('getdata').on('getdata', function(event) {
            event.stopPropagation();
            let data = self.getData();
            self.place.data('[[elementdata]]', data);
        }).off('rendermathquill', '.ql-mathquill').on('rendermathquill', '.ql-mathquill', function(event, data) {
            event.stopPropagation();
            let target = $(event.target);
            self.renderMath(target);
        }).off('setmode').on('setmode', function(event, data) {
            event.stopPropagation();
            // This does not work in Firefox. It hangs the browser for some reason.
            // Instead of reusing this QuillElement in other mode, create a new one.
            //self.setMode(data);
            //self.show();
        });
    };
    
    /**
     * Init event handlers for edit mode
     */
    QuillElement.prototype.initHandlersEdit = function() {
        var self = this;
        this.place.off('quillchanged').on('quillchanged', function(event, data) {
            self.updateData();
            self.delayedChanged();
        }).off('getOutMathquillL').on('getOutMathquillL', function(event, data) {
            event.stopPropagation();
            self.quill.focus();
        }).off('getOutMathquillR').on('getOutMathquillR', function(event, data) {
            event.stopPropagation();
            let quill = self.quill;
            quill.focus();
            let range = quill.getSelection();
            quill.setSelection(range.index + 1, 0);
        });
    }
    
    /**
     * Init event handlers for view mode
     */
    QuillElement.prototype.initHandlersView = function() {
        
    };
    
    /**
     * Render mathquill in given target element.
     * @param {jQuery} target   The element where the MathQuill span is.
     */
    QuillElement.prototype.renderMath = function(target) {
        let mqspan = target.find('.mq-mathfield');
        if (this.editable) {
            mqspan.on('paste', function(event) {
                // Prevent 'paste' event 'leaking' to the Quill editor. MathQuill deals with it just fine.
                event.stopPropagation();
            });
            let mqfield = MQ.MathField(mqspan.get(0));
            let focusable = mqspan.closest('.quillelement-mq-setfocus');
            if (focusable.length > 0) {
                focusable.removeClass('quillelement-mq-setfocus');
                mqfield.focus();
            };
        } else {
            MQ.StaticMath(mqspan.get(0));
        };
    };
    
    /**
     * Get the data of the element
     */
    QuillElement.prototype.getData = function() {
        let result = {
            type: this.type,
            metadata: JSON.parse(JSON.stringify(this.metadata)),
            data: JSON.parse(JSON.stringify(this.data))
        };
        return result;
    };
    
    /**
     * Update the data from the Quill editor
     */
    QuillElement.prototype.updateData = function() {
        let delta = this.quill.getContents();
        if (JSON.stringify(delta) !== JSON.stringify(this.data.delta)) {
            this.data.delta = delta;
        };
    };
    
    /**
     * Trigger element_changed event
     */
    QuillElement.prototype.changed = function() {
        this.metadata.modifier = this.settings.username;
        this.metadata.modified = (new Date()).getTime();
        this.place.trigger('element_changed', {type: this.type});
    };
    
    /**
     * Delayed changed triggering
     */
    QuillElement.prototype.delayedChanged = function() {
        var self = this;
        if (this.changeTimeout) {
            clearTimeout(this.changeTimeout);
            this.changeTimeout = null;
        };
        this.changeTimeout = setTimeout(function() {
            self.changeTimeout = null;
            self.changed();
        }, QuillElement.SAVEDELAY);
    };
    
    /**
     * Add matchers to parse pasted html
     */
    QuillElement.prototype.addMatchers = function() {
        var clipboard = this.quill.clipboard;
        clipboard.addMatcher('.ql-mathquill', function(node, delta) {
            if (node.innerText === '' || node.innerText === '\ufeff') {
                // Firefox splits copied MathQuill (StaticMath) HTML into two .ql-mathquill -spans
                // because of '-moz-user-select: none;' in css.
                // When pasting, we remove empty .ql-mathquill -spans.
                let Delta = Quill.import('delta');
                return new Delta();
            } else {
                return delta;
            };
        });
    };
    
    /**
     * Editing etc. rights for different modes
     */
    QuillElement.prototype.modes = {
        view: {
            editable: false
        },
        edit: {
            editable: true
        }
    };
    
    /**
     * Devault data for the element
     */
    QuillElement.prototype.defaults = {
        type: 'quillelement',
        metadata: {},
        data: {
            delta: {
                ops: []
            }
        },
        settings: {
            mode: 'view',
            username: 'Unknown',
            lang: 'en',
            uilang: 'en'
        }
    };
    
    QuillElement.prototype.styles = `
        <style type="text/css" id="quillelementstyle">
            .quillelement {
                position: relative;
                /*box-shadow: 0 0 0 1px #999;*/
            }
            .quillelement[data-elementmode$="view"] {
                box-shadow: none;
            }
            .quillelement .ql-toolbar {
                position: sticky;
                top: 0;
                background-color: #eee;
                z-index: 5;
            }
            .quillelement[data-elementmode$="view"] .ql-editor {
                overflow-y: inherit;
            }
            .quillelement .ql-editor h1 {
                margin-top: 0.6em;
                font-size: 200%;
            }
            .quillelement .ql-editor h2 {
                margin-top: 1em;
                font-size: 160%;
            }
            .quillelement .ql-editor p {
                margin: 0.5em 0;
            }
            .quillelement .ql-container {
                font-family: inherit;
                font-size: inherit;
            }
            .quillelement .ql-container.ql-snow {
                border: none;
                height: auto;
            }
            .quillelement .ql-container .ql-mathquill .mq-mathfield.mq-editable-field {
                border: 1px solid #ddeeff;
                background-color: #f0faff;
                margin: 0 1px;
            }
        </style>
    `
    
    QuillElement.elementinfo = {
        type: 'quillelement',
        elementtype: ['elements', 'studentelements'],
        jquery: 'quillelement',
        name: 'Quill',
        icon: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-quill"><path style="stroke: none;" d="M2 8 h18 v4 h-1 a2 2 0 0 0 -2 -2 h-4 v14 a2 2 0 0 0 2 2 v1 h-8 v-1 a2 2 0 0 0 2 -2 v-14 h-4 a2 2 0 0 0 -2 2 h-1z m14 18 c2 -2 3 -5 4 -10 c1 -4 3 -12 9 -11 c-1 1 -2 3 -2 5 c-1 1 -1 1 -3 1 c1 1 2 1 3 1 c-0.4 1 -0.4 2 -1 5 c-1 0 -2 0 -3 0.5 c1 0.5 2 0 3 0 c-1 2 -2 5 -6.8 5 l-2.5 4.7z"></path></svg>',
        icon2: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-textelement"><path transform="scale(0.72) translate(1 12)" style="stroke: none;" d="M0 15 l5.625 -15 l4 0 l5.625 15 l-4 0 l-1.5 -4 l-4.25 0 l-1.5 4z m4 0 m1.5 -4 m1 -2.6666 l2.25 0 l-1.125 -3z" /><path transform="scale(0.72) translate(17 12)" style="stroke: none;" d="M0 0 l4 0 a4.2 4 0 0 1 2 7 a2.3 2.5 0 0 1 0 8 l-6 0z m3 2 l0 4 a2 1.5 0 0 0 0 -4z  m0 6 l0 5 a4 2.5 0 0 0 0 -5z" /><path transform="scale(0.8) translate(25 11)" style="stroke: none;" d="M9 0.5 l0 3 a4 4 0 1 0 0 6 l0 3 a6.7 6.7 0 1 1 0 -12z" /></svg>',
        description: {
            en: 'Text content with Quill editor',
            fi: 'Tekstisisältöä Quill-editorilla',
            sv: 'Text med Quill editor'
        },
        roles: ['teacher', 'student', 'author'],
        classes: ['general'],
        weight: 1000
    }
    
    if (typeof($.fn.elementset) == 'function') {
        $.fn.elementset('addelementtype', QuillElement.elementinfo);
    };
    if (typeof($.fn.elementpanel) == 'function') {
        $.fn.elementpanel('addelementtype', QuillElement.elementinfo);
    };
    
})(window, jQuery)
